/* 
 * @(#)Timer 10.5.2002 version 1.0 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */

package tavi;
import javax.swing.*;

/** 
 * Multipurpose timer. The timer runs as a thread. Java virtual machine 
 * updates this thread in periods set by internal variables.
 * 
 * @version     1.0 10 May 2002 
 * @author	Kupiainen Esko 
 */ 
public class Timer extends Thread {

    /**
     * Timeout in milliseconds.
     */
    private int timeOut = 0;

    /**
     * Current time in milliseconds at countdown start.
     */
    private long startTime = 0;

    /**
     * Reference to message window to be closed after timeout.
     */
    private JDialog messageWindow;

    /**
     * Sleep time in milliseconds between Java vm visits.
     */
    private int sleepMilliSeconds = 100;

    /**
     * Constructor creates a new timer.
     * @param Name of the object. Any String object is valid here.
     */
    public Timer(String name) {

        /* Create a new timer*/
        super(name); 
    }

    /**
     * Java vm runs this method at thread startup.
     */
    public void run() {

        /* once started, run the loop continuously*/
        while(true) {
	    /*close the message window when countdown is finished*/
	    if (System.currentTimeMillis() - startTime > timeOut &&
		messageWindow != null) {
		messageWindow.setVisible(false);
	    	messageWindow = null;
	    } 
	 
	    try {
	        /* request for Java vm to activate this loop again after 
		   a pause of sleepMilliSeconds*/
  	        sleep(sleepMilliSeconds);
	    }
	    catch(InterruptedException e) { 
	        System.out.println("Timer: Error in timeout counting."+ e);
	    }
        } 
    }

    /**
     * Initiates timeout counting.
     * @param timeout Timeout in milliseconds.
     * @param window Reference to window to be closed. 
     */
    public void startCountDown(int timeout, JDialog window) {

	/*set timeout value in milliseconds*/
	timeOut = timeout;

	/*set start time in milliseconds*/
	startTime = System.currentTimeMillis();

	/*set reference to window to be closed*/
	messageWindow = window;
    }

    /**
     * Returns info whether timeout counting is in progress.
     * @return True if timeout counting is in progress, false otherwise. 
     */
    public boolean isCountDownRunning() {
	return System.currentTimeMillis() - startTime < timeOut;
    }

    /**
     * Interrupts countdown process.
     * Disposes message window if open.
     */
    public void stopCountDown() {
	if (messageWindow != null) {
	    messageWindow.setVisible(false);
	    messageWindow = null;
	}
	startTime = 0;
    }
}
