/* 
 * @(#)ServerTest 18.2.2002 version 0.1.2 
 * 
 * Copyright University of Helsinki / Department of computer science 
 * 
 */ 

package tavi.graph;

import junit.framework.*;
import java.awt.*;

/** 
 * JUnit test-class for Server.java. 
 * 
 * @version     0.1.2 04 Mar 2002 
 * @author      Johannes Ukkonen
 * @author      Janne Petj
 */ 
public class ServerTest extends TestCase {
    /**
     * Standard JUnit testfile constructor.
     * @param name Name of the test (should always supply the name of the
     *        class as per JUnit doctrine).
     */
    public ServerTest(String name) {
        super(name);
    }
    /**
     * Standard JUnit command line launcher. Allows GraphTest to be 
     * executed independently from command line, type 'java ServerTest'.
     * @param args[] Required in main method. Not used.
     */
    public static void main(String[] args) {
        junit.swingui.TestRunner.run(ServerTest.class);
    }
    /**
     * Automatically executed before every test. Common testcase 
     * initialization should be put here.
     */
    protected void setUp() {
	;
    }
    /**
     * Complement of setUp(). Common testcase zeroeing should be 
     * put here.
     */
    protected void tearDown() {
        ;
    }
    /**
     * Test method Server().
     */
    public void testServer() {
	Server x = new Server(new Point(3,7), 8, "ab");
	assertTrue(x.getLocation().x == 3 && x.getLocation().y == 7);
	assertTrue(x.queueLength() == 8);
	assertTrue((x.getIdentifier()).equals("ab"));
	/* Test that begin and end states are assigned correctly. */
	assertTrue(!x.isBegin());
	assertTrue(!x.isEnd());
	Server qLess = new Server(new Point(8, 1), 'B');
	assertTrue(qLess.isBegin());
	assertTrue((qLess.getIdentifier()).equals("B"));
	Server qPid = new Server(new Point(900, 31), 'E');
	assertTrue(qPid.isEnd());
	assertTrue((qPid.getIdentifier()).equals("E"));

	/* Test that servers' names are assigned correctly. */
	assertTrue((x.getName()).equals(""));
	x.setServerName("money for nothing");
	assertTrue((x.getName()).equals("money for nothing"));
	x.setServerName(null);
	assertTrue(x.getName() != null);

	/* Check adding distance. */
	assertTrue(x.checkLocation(new Point(90, 101)));
    } 

    /**
     * Tests method isSizeChangeOk.
     */
    public void testIsSizeChangeOk() {
	Server a = new Server(new Point(10,20), 8, "ab");
	Server b = new Server(new Point(10, 10), 1, "ac");	
	TFlow t1 = new TFlow(a, b, 1, 975, null);
	TFlow t2 = new TFlow(b, b, 1, 4, null);
	int[] xP1 = {31, 35};
	int[] yP1 = {4, 5};
	int[] xP2 = {4, 5};
	int[] yP2 = {31, 35};

	/* If there aren't any TFlows in draw area, this method should 
	   stop immediately */ 
	assertTrue(a.isSizeChangeOk(1, 1));

	a.addTFlow(t1);
	b.addTFlow(t2);
	t1.setXTurnPoints(xP1);
	t1.setYTurnPoints(yP1);
	t2.setXTurnPoints(xP2);
	t2.setYTurnPoints(yP2);
	
	/* Tests that size change doesn't leave any routing points outside 
	   draw area */
	assertTrue(a.isSizeChangeOk(401, 336));
	assertTrue(!a.isSizeChangeOk(1, 6));
	assertTrue(!a.isSizeChangeOk(31, 336));
	assertTrue(a.isSizeChangeOk(36, 9));
	assertTrue(b.isSizeChangeOk(6, 36));
	
	TFlow t3 = new TFlow(a, a, new Point(25, 20), 1, 1, null);
	TFlow t4 = new TFlow(a, a, new Point(10, 45), 1, 1, null);
	a.addTFlow(t3);
	a.addTFlow(t4);
	t3.drawSelf(null);
	t4.drawSelf(null);

	/* Tests that recursive tflow is inside requested boundaries. 
	   For testing, we use selection center, which is the furthest 
	   point in recursive TFlow. */
	assertTrue(!a.isSizeChangeOk(35, 300));
	assertTrue(!a.isSizeChangeOk(40, 300));
	assertTrue(!a.isSizeChangeOk(t3.getSelectionCenter().x + 4, 300));
	assertTrue(a.isSizeChangeOk(t3.getSelectionCenter().x + 5, 300));
	assertTrue(!a.isSizeChangeOk(300, 45));
	assertTrue(!a.isSizeChangeOk(300, 40));
	assertTrue(!a.isSizeChangeOk(300, t4.getSelectionCenter().y + 4));
	assertTrue(a.isSizeChangeOk(300, t4.getSelectionCenter().y + 5));
    }

    /**
     * Tests method addTFlow.
     */
    public void testAddTFlow() {
	Server a = new Server(new Point(3,7), 8, "ab");
	Server b = new Server(new Point(), 1, "ac");
	assertTrue(a.edges.size() == 0);
	TFlow t = new TFlow(a, b, 1, 975, null);
	a.addTFlow(t);
	assertTrue(a.edges.size() == 1);
	for (int i = 1; i < 10; ++i) {
	    t = new TFlow(a, b, i, 975, null);
	    a.addTFlow(t);
	    assertTrue(a.edges.size() == i+1);
	}
    }

    /**
     * Tests method adjustRouted.
     */
    public void testAdjustRouted() {
    Server a = new Server(new Point(3,7), 8, "ab");
    Server b = new Server(new Point(), 1, "ac");
    TFlow t = new TFlow(a, b, 1, 975, null);
    a.addTFlow(t);
    int[] xTp = new int [3];
    int[] yTp = new int [3];
    t.setXTurnPoints(xTp);
    t.setYTurnPoints(yTp);
    a.adjustRouted(12, 13);
    xTp = t.getXTurnPoints();
    yTp = t.getYTurnPoints();
    assertTrue(xTp[0] == 12 && xTp[1] == 12 && xTp[2] == 12);
    assertTrue(yTp[0] == 13 && yTp[1] == 13 && yTp[2] == 13);
    }
    
    /**
     * Tests method arcsInThisDirection.
     */
    public void testArcsInThisDirection() {
	Server a = new Server(new Point(100, 100), 8, "ab");
	TFlow t = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			    1, 1, null);
	TFlow t1 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	a.addTFlow(t);
	a.addTFlow(t1);
	assertTrue(a.arcsInThisDirection(2) == 2);
	assertTrue(a.arcsInThisDirection(1) == 0);
	assertTrue(a.arcsInThisDirection(0) == 0);
	assertTrue(a.arcsInThisDirection(666) == 0);
	TFlow t2 = new TFlow(a, a, new Point(100 + (Server.radius * 3/4), 100),
			     1, 1, null);
	TFlow t3 = new TFlow(a, a, new Point(100, 100 - (Server.radius * 3/4)),
			     1, 1, null);
	TFlow t4 = new TFlow(a, a, new Point(100 - (Server.radius * 3/4), 100),
			     1, 1, null);
	a.addTFlow(t2);
	a.addTFlow(t3);
	a.addTFlow(t4);
	assertTrue(a.arcsInThisDirection(2) == 2);
	assertTrue(a.arcsInThisDirection(1) == 1);
	assertTrue(a.arcsInThisDirection(0) == 1);
	assertTrue(a.arcsInThisDirection(3) == 1);
	TFlow t5 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	TFlow t6 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	TFlow t7 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	TFlow t8 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	a.addTFlow(t5);
	a.addTFlow(t6);
	a.addTFlow(t7);
	a.addTFlow(t8);
	assertTrue(a.arcsInThisDirection(2) == 6);
	assertTrue(a.arcsInThisDirection(1) == 1);
	assertTrue(a.arcsInThisDirection(0) == 1);
	assertTrue(a.arcsInThisDirection(3) == 1);
    }
    
    /**
     * Tests method that checks that location of mouse click is at least
     * minimum distance away from objects.
     */
    public void testCheckLocation() {
	Server a = new Server(new Point(100, 100), 8, "ab");
	Server b = new Server(new Point(200, 200), 1, "ac");

	/* Construct new frame because some information is calculated
	   in the drawSelf method of TFlow the earliest. */
	Frame fr = new Frame();
	fr.pack();
	fr.setVisible(true);
	Graphics testGraphics = fr.getGraphics();
	fr.dispose();
	a.drawSelf(testGraphics, null);
	b.drawSelf(testGraphics, null);

	assertTrue(a.checkLocation(new Point(150, 150)));	
    
	TFlow t = new TFlow(a, b, 1, 975, null);
	a.addTFlow(t);
	t.drawSelf(null);
	assertTrue(!a.checkLocation(new Point(150, 150)));
	assertTrue(!a.checkLocation(new Point(200, 150)));
	assertTrue(a.checkLocation(new Point(201, 99)));
	assertTrue(a.checkLocation(new Point(201, 100)));
	assertTrue(a.checkLocation(new Point(201, 125)));
	assertTrue(a.checkLocation(new Point(201, 126)));
    }

    /**
     * Test method isConnected().
     */
    public void testIsConnected() {
	/* test single server graph */
	Server a = new Server(new Point(), 0, "ab");
	Server b = new Server(new Point(), 1, "ac");
	assertTrue(!a.isConnected(b));
	assertTrue(!b.isConnected(a));
	a.addTFlow(new TFlow(a, b, 1, 975, null));
	assertTrue(a.isConnected(b));
	assertTrue(b.isConnected(a));

	Server[] aList = new Server[5];

	for (int i = 0; i < aList.length; ++i) {
	    aList[i] = new Server(new Point(), i,"ad");
	}
	for (int i = 0; i < aList.length; ++i) {
	    for (int j = 0; j < aList.length; ++j) {
		assertTrue(!(aList[i].isConnected(aList[j])));
	    }
	}
	/* all but first are connected to next in line, and third is 
	   connected to fifth (eg.  0  1->2->3->4 2->4 )*/
	for (int i = 1; i < aList.length - 1; ++i) {
	    aList[i].addTFlow(new TFlow(aList[i], aList[i+1], 1, 2, null));
	}
	aList[2].addTFlow(new TFlow(aList[2], aList[4], 1, 990, null));

	/* assert that connections are ok */
	for (int i = 0; i < aList.length; ++i) {
	    for (int j = 0; j < aList.length; ++j) {
		if ((i == 1 && j == 2) || (i == 2 && j == 3) ||
		    (i == 3 && j == 4) || (i == 2 && j == 4) ||
		    (j == 1 && i == 2) || (j == 2 && i == 3) ||
		    (j == 3 && i == 4) || (j == 2 && i == 4)) {		  
		    assertTrue((aList[i].isConnected(aList[j])));
		} else {
		    assertTrue(!(aList[i].isConnected(aList[j])));
		}
	    }
	}    
    }  


    /**
     * Test method edgesTo(Server s).
     */
    public void testEdgesTo() {
	Server[] aList = new Server[3];
	TFlow[] flowList = new TFlow[6];
	for (int i = 0; i < aList.length; ++i) {
	    aList[i] = new Server(new Point(i * 100, i * 100), i, "ab");
	}
	/* initially there should be no connections */
	assertTrue(aList[1].edgesTo(aList[2]) == 0);	
	/* create an edge between second and third server */
	flowList[0] = new TFlow(aList[1], aList[2], 1, 111, null);
	aList[1].addTFlow(flowList[0]);
	/* there should be one connection between second and third */
	assertTrue(aList[1].edgesTo(aList[2]) == 1);
	/* create a recursion edge to second server, a client 
	   routed edge between second and third, a normal edge
	   between third and second,a normal edge between second 
	   and first and a normal edge between second and third server */
	flowList[1] = new TFlow(aList[1], aList[1], new Point(100, 120), 1, 34,
				null);
	flowList[2] = new TFlow(aList[1], aList[2], 1, 543, null);
	flowList[2].setXTurnPoints(new int[1]);
	flowList[2].setYTurnPoints(new int[1]);	 
	flowList[3] = new TFlow(aList[1], aList[0], 1, 444, null);
	flowList[4] = new TFlow(aList[2], aList[1], 1, 553, null);
	flowList[5] = new TFlow(aList[1], aList[2], 1, 545, null);
	aList[1].addTFlow(flowList[1]);
	aList[1].addTFlow(flowList[2]);
	aList[1].addTFlow(flowList[3]);
	aList[1].addTFlow(flowList[4]);
	aList[1].addTFlow(flowList[5]);
	/* the recursion edge and routed edge, and the normal edges
	   between third and second and second and first should not 
	   show in the calculation */
	assertTrue(aList[1].edgesTo(aList[2]) == 2);	
    }

    /**
     * Tests that the identifier can be correctly fetched.
     */
    public void testGetIdentifier() {
	Server a = new Server(new Point(), 0, "ab");
	assertTrue(a.getIdentifier().equals("ab"));
	Server begin = new Server(new Point(), 'B');
	assertTrue(begin.getIdentifier().equals("B"));
    }
    
    /**
     * Tests that the location of the server can be correctly fetched.
     */
    public void testGetLocation() {
	Server a = new Server(new Point(2, 5), 0, "ab");
	assertTrue(a.getLocation().x == 2);
	assertTrue(a.getLocation().y == 5);	
	Server b = new Server(new Point(-2, -666), 0, "ab");
	assertTrue(b.getLocation().x == -2);
	assertTrue(b.getLocation().y == -666);
	Server begin = new Server(new Point(198565, 7777777), 'B');
	assertTrue(begin.getLocation().x == 198565);
	assertTrue(begin.getLocation().y == 7777777);
    }
        
    /**
     * Tests that the (long) name of the server can be correctly fetched.
     */
    public void testGetName() {
	Server a = new Server(new Point(2, 5), 0, "ab");
	a.setServerName("Joojoo-serveri");
	assertTrue(a.getName().equals("Joojoo-serveri"));	
	Server b = new Server(new Point(-2, -666), 0, "ab");
	b.setServerName("Oikea serveri");
	assertTrue(b.getName().equals("Oikea serveri"));
    }

    /**
     * Tests method getRecursiveMax.
     */
    public void testGetRecursiveMax() {
	Server a = new Server(new Point(100, 100), 8, "ab");
	Server b = new Server(new Point(200, 200), 1, "ac");
	TFlow t1 = new TFlow(a, b, 1, 975, null);
	TFlow t2 = new TFlow(a, a, new Point(100, 80), 2, 975, null);
	TFlow t3 = new TFlow(a, a, new Point(100, 80), 4, 975, null);
	TFlow t4 = new TFlow(a, a, new Point(100, 80), 7, 975, null);
	TFlow t5 = new TFlow(a, a, new Point(100, 80), 9000, 975, null);
	a.addTFlow(t1);
	a.addTFlow(t2);
	a.addTFlow(t3);
	a.addTFlow(t4);
	a.addTFlow(t5);
	assertTrue(a.getRecursiveMax() == 4);
    }

    /**
     * Tests method getServerMinMax.
     */
    public void testGetServerMinMax() {
	Server a = new Server(new Point(100, 100), 8, "ab");
	Server b = new Server(new Point(200, 200), 1, "ac");
	TFlow t1 = new TFlow(a, b, 1, 975, null);
	TFlow t2 = new TFlow(a, a, new Point(100, 80), 2, 975, null);
	TFlow t3 = new TFlow(b, b, new Point(220, 200), 4, 975, null);
	TFlow t4 = new TFlow(b, b, new Point(200, 220), 7, 975, null);
	a.addTFlow(t1);
	a.addTFlow(t2);
	b.addTFlow(t3);
	b.addTFlow(t4);
	int [] minMaxA = a.getServerMinMax((double)1);
	assertTrue(minMaxA[0] > 53 && minMaxA[0] < 57);
	assertTrue(minMaxA[1] > 44 && minMaxA[1] < 48);
	assertTrue(minMaxA[2] > 152 && minMaxA[2] < 156);
	assertTrue(minMaxA[3] > 160 && minMaxA[3] < 164);

	int [] minMaxB = b.getServerMinMax((double)1);
	assertTrue(minMaxB[0] > 153 && minMaxB[0] < 157);
	assertTrue(minMaxB[1] > 164 && minMaxB[1] < 169);
	assertTrue(minMaxB[2] > 251 && minMaxB[2] < 256);
	assertTrue(minMaxB[3] > 251 && minMaxB[3] < 256);
    }
   

    /*
     * Tests that the nearest edge of choosed point is returned. 
     * Null if none is near.
     * CAN'T BE TESTED SO FAR! (Because of TFlow...) 
    public void testGetClickedEdge() {
	Server a = new Server(new Point(2, 5), 0, "ab");
	
    }
    */
    
    /**
     * Tests that Begin-server can be correctly fetched
     */
    public void testIsBegin() {
	Server b = new Server(new Point(-2, -666), 0, "ab");
	assertTrue(!b.isBegin());
	Server begin = new Server(new Point(198565, 7777777), 'B');
	assertTrue(begin.isBegin());
	Server end = new Server(new Point(198565, 7777777), 'E');
	assertTrue(!end.isBegin());
    }

    /**
     * Tests that End-server can be correctly fetched
     */
    public void testIsEnd() {
	Server b = new Server(new Point(-2, -666), 0, "ab");
	assertTrue(!b.isEnd());
	Server begin = new Server(new Point(198565, 7777777), 'B');
	assertTrue(!begin.isEnd());
	Server end = new Server(new Point(198565, 7777777), 'E');
	assertTrue(end.isEnd());
    }
    
    /**
     * Tests that method which returns server's queue size works correct.
     */
    public void testQueueLength() {
	Server b = new Server(new Point(-2, -666), 0, "ab");
	assertTrue(b.queueLength() == 0);
	Server b2 = new Server(new Point(198565, 7777777), 77, "cd");
	assertTrue(b2.queueLength() == 77);
	Server e = new Server(new Point(198565, 7777777), 666, "xx");
	assertTrue(e.queueLength() == 666);
	Server e2 = new Server(new Point(198565, 7777777), -666, "xx");
	assertTrue(e2.queueLength() == 0);
    }

    /**
     * Tests method recArcsBefore.
     */
    public void testRecArcsBefore() {
	Server a = new Server(new Point(100,100), 8, "ab");
	Server b = new Server(new Point(200, 200), 6, "hh");
	TFlow t = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			    1, 1, null);
	TFlow t1 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	a.addTFlow(t);
	a.addTFlow(t1);
	assertTrue(a.recArcsBefore(t1) == 1);
	TFlow t2 = new TFlow(a, a, new Point(100 + (Server.radius * 3/4), 100),
			     1, 1, null);
	TFlow t3 = new TFlow(a, a, new Point(100, 100 - (Server.radius * 3/4)),
			     1, 1, null);
	TFlow t4 = new TFlow(a, a, new Point(100 - (Server.radius * 3/4), 100),
			     1, 1, null);
	a.addTFlow(t2);
	a.addTFlow(t3);
	a.addTFlow(t4);
	assertTrue(a.recArcsBefore(t1) == 1);
	assertTrue(a.recArcsBefore(t2) == 0);
	assertTrue(a.recArcsBefore(t3) == 0);
	assertTrue(a.recArcsBefore(t4) == 0);
	assertTrue(a.recArcsBefore(t) == 0);
	TFlow t5 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	TFlow t6 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	TFlow t7 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	TFlow t8 = new TFlow(a, a, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	a.addTFlow(t5);
	a.addTFlow(t6);
	a.addTFlow(t7);
	a.addTFlow(t8);
	assertTrue(a.recArcsBefore(t8) == 5);
	assertTrue(a.recArcsBefore(t7) == 4);
	assertTrue(a.recArcsBefore(t) == 0);
	assertTrue(a.recArcsBefore(t1) == 1);
	assertTrue(a.recArcsBefore(t4) == 0);
	assertTrue(a.recArcsBefore(t5) == 2);
	TFlow t9 = new TFlow(b, b, new Point(100, 100 + (Server.radius * 3/4)),
			     1, 1, null);
	b.addTFlow(t9);
	assertTrue(a.recArcsBefore(t9) == 0);
	assertTrue(a.recArcsBefore(null) == 0);
    }  

    /**
     * Tests method setQueueLength
     */
    public void testSetQueueLength() {
	Server a = new Server(new Point(3,7), 8, "ab");
	a.setQueueLength(7);
	assertTrue(a.queueLength() == 7);
	a.setQueueLength(0);
	assertTrue(a.queueLength() == 0);
	a.setQueueLength(-5);
	assertTrue(a.queueLength() == 0);

    }

    /**
     * Tests method setSelected
     */
    public void testSetSelected() {
	Server a = new Server(new Point(3,7), 8, "ab");
	assertTrue(a.selected == false);
	a.setSelected(true);
	assertTrue(a.selected == true);
	a.setSelected(false);
	assertTrue(a.selected == false);
    }

    /**
     * Tests method setServerName
     */
    public void testSetServerName() {
	Server a = new Server(new Point(3,7), 8, "ab");
	a.setServerName("uusi-serveri");
	assertTrue(a.getName() == "uusi-serveri");
	a.setServerName(null);
	assertTrue(a.getName() == "uusi-serveri");
    }

    /**
     * Tests method removeTFlow.
     */
    public void testRemoveTFlow() {
	Identifier id = new Identifier();
	Server a = new Server(new Point(3,7), 8, "ab");
	Server b = new Server(new Point(), 1, "ac");
	TFlow t = new TFlow(a, b, 1, 975, null);
	assertTrue(!a.removeTFlow(t, id));
	a.addTFlow(t);
	assertTrue(a.removeTFlow(t, id));
	assertTrue(!a.removeTFlow(t, id));
	TFlow t1 = new TFlow(a, b, 2, 975, null);
	TFlow t2 = new TFlow(a, b, 4, 975, null);
	TFlow t3 = new TFlow(a, b, 7, 975, null);
	TFlow t4 = new TFlow(a, b, 9000, 975, null);
	a.addTFlow(t1);
	a.addTFlow(t2);
	a.addTFlow(t3);
	a.addTFlow(t4);
	assertTrue(a.removeTFlow(t1, id));
	assertTrue(a.removeTFlow(t2, id));
	assertTrue(a.removeTFlow(t3, id));
	assertTrue(a.removeTFlow(t4, id));
	assertTrue(!a.removeTFlow(null, id));	
    }

    /**
     * Tests method which eventually changes TFlow's direction.
     */
    public void testSwapOwners() {
	Server a = new Server(new Point(3,7), 8, "ab");
	Server b = new Server(new Point(), 1, "ac");
	TFlow t = new TFlow(a, b, 1, 975, null);
	a.addTFlow(t);
	assertTrue(t.getSource().equals(a));
	a.swapOwners(t);
	assertTrue(t.getSource().equals(b));
	assertTrue(!t.getSource().equals(a));
	TFlow t1 = new TFlow(a, a, 2, 975, null);
	a.addTFlow(t1);
	assertTrue(t1.getSource().equals(a));
	a.swapOwners(t1);
	assertTrue(t1.getSource().equals(a));
	TFlow t2 = new TFlow(a, b, 4, 975, null);
	a.addTFlow(t2);
	assertTrue(t2.getSource().equals(a));
	a.swapOwners(t2);
	assertTrue(t2.getSource().equals(b));
	assertTrue(!t2.getSource().equals(a));
	assertTrue(t2.getDestination().equals(a));
	assertTrue(!a.swapOwners(null));	
    }
    
    /**
     * Tests method removeConnectedTFlows.
     */
    public void testRemoveConnectedTFlows() {
	Identifier id = new Identifier();
	Server a = new Server(new Point(3,7), 8, "ab");
	assertTrue(a.edges.isEmpty());
	Server b = new Server(new Point(), 1, "ac");
	TFlow t = new TFlow(a, b, 1, 975, null);
	a.addTFlow(t);
	assertTrue(!a.edges.isEmpty());
	TFlow t1 = new TFlow(a, b, 2, 975, null);
	TFlow t2 = new TFlow(a, b, 4, 975, null);
	TFlow t3 = new TFlow(a, b, 7, 975, null);
	TFlow t4 = new TFlow(a, b, 9000, 975, null);
	a.addTFlow(t1);
	a.addTFlow(t2);
	a.addTFlow(t3);
	a.addTFlow(t4);
	a.removeConnectedTFlows(b, id);
	assertTrue(a.edges.isEmpty());
	TFlow t5 = new TFlow(a, b, 2, 975, null);
	TFlow t6 = new TFlow(a, b, 4, 975, null);
	TFlow t7 = new TFlow(a, a, 7, 975, null);
	a.addTFlow(t5);
	a.addTFlow(t6);
	a.addTFlow(t7);
	assertTrue(!a.edges.isEmpty());
	a.removeConnectedTFlows(b, id);
	assertTrue(!a.edges.isEmpty());
    }

    /**
     * Tests method that renumbers edges.
     */
    public void testRenumberEdges() {
	Identifier id = new Identifier();
	Server a = new Server(new Point(3,7), 8, "ab");
	Server b = new Server(new Point(), 1, "ac");
	TFlow t = new TFlow(a, b, 1, 1, null);
	a.addTFlow(t);
	TFlow t1 = new TFlow(a, b, 2, 2, null);
	a.addTFlow(t1);
	assertTrue(t.getOrder() == 1);
	assertTrue(t1.getOrder() == 2);
	a.renumberEdges(a, b);
	assertTrue(t.getOrder() == 1);
	assertTrue(t1.getOrder() == 2);
	b.renumberEdges(b, a);
	assertTrue(t.getOrder() == 1);
	assertTrue(t1.getOrder() == 2);
	a.removeTFlow(t, id);
	a.renumberEdges(a, b);
	assertTrue(t1.getOrder() == 1);
	TFlow t2 = new TFlow(b, a, 4, 2, null);
	TFlow t3 = new TFlow(a, b, 25, 2, null);
	TFlow t4 = new TFlow(a, b, 26, 2, null);
	TFlow t5 = new TFlow(b, a, 27, 2, null);
	TFlow t6 = new TFlow(a, b, 28, 2, null);
	b.addTFlow(t2);
	a.addTFlow(t3);
	a.addTFlow(t4);
	b.addTFlow(t5);
	a.addTFlow(t6);
	a.renumberEdges(a, b);
	assertTrue(t1.getOrder() == 1);
	assertTrue(t3.getOrder() == 2);
	assertTrue(t4.getOrder() == 3);
	assertTrue(t6.getOrder() == 4);
	assertTrue(t2.getOrder() == 5);
	assertTrue(t5.getOrder() == 6);
	b.removeTFlow(t2, id);
	a.renumberEdges(a, b);
	assertTrue(t1.getOrder() == 1);
	assertTrue(t3.getOrder() == 2);
	assertTrue(t4.getOrder() == 3);
	assertTrue(t6.getOrder() == 4);
	assertTrue(t5.getOrder() == 5);
	a.removeTFlow(t1, id);
	a.renumberEdges(a, b);
	assertTrue(t3.getOrder() == 1);
	assertTrue(t4.getOrder() == 2);
	assertTrue(t6.getOrder() == 3);
	assertTrue(t5.getOrder() == 4);
    }
}
