/* 
 * @(#)TFlowTest 25.3.2002 version 0.1.5
 * 
 * Copyright University of Helsinki / Department of computer science 
 *
 */ 

package tavi.graph;

import junit.framework.*;
import java.awt.*;
import java.util.*;

/** 
 * JUnit test-class for TFlow.java.
 * Tests expect: Server.radius = 25. 
 * 
 * @version     0.1.5 25 Mar 2002 
 * @author      Johannes Ukkonen
 * @author      Esko Kupiainen
 * @author      Kimmo Airamaa
 */ 
public class TFlowTest extends TestCase {

    private Graphics testGraphics;
    private Transaction transact;

    /**
     * Standard JUnit testfile constructor.
     * @param name Name of the test (should always supply the name of the
     *        class as per JUnit doctrine).
     */
    public TFlowTest(String name) {
        super(name);
	transact = new Transaction ("Test", "None", new Identifier());
    }

    /**
     * Standard JUnit command line launcher. Allows GraphTest to be 
     * executed independently from command line, type 'java TFlowTest'.
     * @param args[] Required in main method. Not used.
     */
    public static void main(String[] args) {
        junit.swingui.TestRunner.run(TFlowTest.class);
    }

    /**
     * Automatically executed before every test. Common testcase 
     * initialization should be put here.
     */
    protected void setUp() {
	;
    }

    /**
     * Complement of setUp(). Common testcase zeroeing should be 
     * put here 
     */
    protected void tearDown() {
        ;
    }

    /**
     * Test method that handles the testing of changeDirection method.
     */
    public void testChangeDirection() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	
	/* Most simple case: 'a straight line'. */
	TFlow t = new TFlow(a, b, 1, 1, transact);	
	t.changeDirection();
	assertTrue(t.getSource() == b);
	assertTrue(t.getDestination() == a);
	t.changeDirection();
	assertTrue(t.getSource() == a);
	assertTrue(t.getDestination() == b);
    }

    /**
     * Test method for isRouted.
     */ 
    public void testIsRouted() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1, transact);	
	TFlow flow = new TFlow(a, a, 1, 1, transact);
	assertTrue(!t.isRouted());
	assertTrue(!flow.isRouted());
	TFlow tflow = new TFlow(a, b, 1, 1,transact);
	int[] xPoints = {345, 346};
	int[] yPoints = {345, 64};
	tflow.setXTurnPoints(xPoints);
	tflow.setYTurnPoints(yPoints);
	assertTrue(tflow.isRouted());
    }

    /**
     * Test method for setScheme.
     */
    public void testSetScheme() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1,transact );	

	String scheme = "sufficient scheme.";
	t.setScheme(scheme);
	assertTrue((t.getScheme()).equals(scheme));
	t.setScheme(null);
	assertTrue((t.getScheme()).equals(scheme));
    }

    /**
     * Test method for getScheme.
     */
    public void testGetScheme() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1, transact);	

	String scheme = "sufficient scheme.";
	t.setScheme(scheme);
	assertTrue((t.getScheme()).equals(scheme));
	t.setScheme(null);
	assertTrue((t.getScheme()).equals(scheme));
    }

    
    /**
     * Test method for setName.
     */
    public void testSetName() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1, transact);
	
	assertTrue((t.getName()).equals(""));
	String name = "Very informal name.";
	t.setName(name);
	assertTrue((t.getName()).equals(name));
	t.setName(null);
	assertTrue((t.getName()) != null);
    }

    /**
     * Test method for setName.
     */
    public void testGetName() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1,transact );
	
	assertTrue((t.getName()).equals(""));
	String name = "Very informal name.";
	t.setName(name);
	assertTrue((t.getName()).equals(name));
	t.setName(null);
	assertTrue((t.getName()) != null);
    }

    /**
     * Test method for getIdentifier.
     */
    public void testGetIdentifier() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t1 = new TFlow(a, b, 0, 1, transact);
	TFlow t2 = new TFlow(a, b, 1, 1, transact);
	TFlow t3 = new TFlow(a, b, 2, 1, transact);
	
	assertTrue(t1.getIdentifier() == 0);
	assertTrue(t2.getIdentifier() == 1);
	assertTrue(t3.getIdentifier() == 2);
    }
    

    /**
     * Method that tests that the TFlow is selected and deselected right.
     */
    public void testSetSelected() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1, transact);
	
	assertTrue(!t.getSelected());
	t.setSelected(true);
	assertTrue(t.getSelected());
	t.setSelected(false);
	assertTrue(!t.getSelected());
    }

    /**
     * Tests the setDestination method.
     */
    public void testSetDestination() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1, transact);
	
	assertTrue((t.getDestination()) == b);
	t.setDestination(a);
	assertTrue((t.getDestination()) == a);
	t.setDestination(null);
	assertTrue((t.getDestination()) != null);
    }

    /**
     * Tests that the selection area is displayed when it is supposed to.
     */
    public void testSetShowSelectionArea() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1,transact );

	assertTrue(t.getShowSelectionArea());
	t.setShowSelectionArea(false);
	assertTrue(!(t.getShowSelectionArea()));
	t.setShowSelectionArea(true);
    }

    /**
     * Tests the getSelectionCenter method
     */
    public void testGetSelectionCenter() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t = new TFlow(a, b, 1, 1, transact);

	Graph graph = new Graph();
        graph.addTFlow (a, b, new Point (0, 0), 1,1);

	t.drawSelf(null);
	assertTrue(t.getSelectionCenter().x == 150);
	assertTrue(t.getSelectionCenter().y == 150);
    }

    /**
     * Tests the getSelectionAreaRadius method
     */
    public void testGetSelectionAreaRadius() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t = new TFlow(a, b, 1, 1, transact);
	t.drawSelf(null);
	assertTrue(t.getSelectionAreaRadius() == Server.getRadius() * 4 / 25);
    }

    /**
     * Test method for getXTurnPoints.
     */
    public void testGetXTurnPoints() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1, transact);
	int [] xTp = {1, 100, 200};
	t.setXTurnPoints(xTp);
	assertTrue(t.getXTurnPoints() != null);
	assertTrue(t.getXTurnPoints()[0] == 1);
	assertTrue(t.getXTurnPoints()[1] == 100);
	assertTrue(t.getXTurnPoints()[2] == 200);
    }

    /**
     * Test method for getYTurnPoints.
     */
    public void testGetYTurnPoints() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1, transact);
	int [] yTp = {100, 10, 2000};
	t.setYTurnPoints(yTp);
	assertTrue(t.getYTurnPoints() != null);
	assertTrue(t.getYTurnPoints()[0] == 100);
	assertTrue(t.getYTurnPoints()[1] == 10);
	assertTrue(t.getYTurnPoints()[2] == 2000);
    }

    /**
     * Test method for setXTurnPoints.
     */
    public void testSetXTurnPoints() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1,transact );
	int [] xTp = {-1, 100, -200};
	t.setXTurnPoints(xTp);
	assertTrue(t.getXTurnPoints() != null);
	assertTrue(t.getXTurnPoints()[0] == -1);
	assertTrue(t.getXTurnPoints()[1] == 100);
	assertTrue(t.getXTurnPoints()[2] == -200);
    }

    /**
     * Test method for setYTurnPoints.
     */
    public void testSetYTurnPoints() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1, transact);
	int [] yTp = {-100, 10, -2000};
	t.setYTurnPoints(yTp );
	assertTrue(t.getYTurnPoints() != null);
	assertTrue(t.getYTurnPoints()[0] == -100);
	assertTrue(t.getYTurnPoints()[1] == 10);
	assertTrue(t.getYTurnPoints()[2] == -2000);
    }

    /**
     * Test method for getSource.
     */
    public void testGetSource() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1,transact);
	assertTrue(t.getSource() != null);
	assertTrue(t.getSource().equals(a));
    }

    /**
     * Test method for getDestination.
     */
    public void testGetDestination() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1,transact);
	assertTrue(t.getDestination() != null);
	assertTrue(t.getDestination().equals(b));
    }

    /**
     * Test method for setSource.
     */
    public void testSetSource() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");	
	TFlow t = new TFlow(a, b, 1, 1,transact);
	assertTrue(t.getSource() != null);
	assertTrue(t.getSource().equals(a));
	t.setSource(b);
	assertTrue(t.getSource().equals(b));
    }

    /**
     * Test method for getSegmentRadius.
     */
    public void testGetSegmentRadius() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t1 = new TFlow(a, b, 1, 1,transact);
	TFlow t2 = new TFlow(b, a, 1, 2,transact);
        a.addTFlow (t1);
	b.addTFlow (t2);
	t1.drawSelf(null);
	t2.drawSelf(null);
	assertTrue(t1.getSegmentRadius() > 586 && t1.getSegmentRadius() < 588);
 	assertTrue(t2.getSegmentRadius() > 586 && t2.getSegmentRadius() < 588);
    }


    /**
     * Test method for getSegmentCenterX.
     */
    public void testGetSegmentCenterX() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t1 = new TFlow(a, b, 1, 1,transact);
	TFlow t2 = new TFlow(b, a, 1, 2,transact);
        a.addTFlow (t1);
	b.addTFlow (t2);
	t1.drawSelf(null);
	t2.drawSelf(null);
	assertTrue(t1.getSegmentCenterX() > 560 && t1.getSegmentCenterX() < 562);
	assertTrue(t2.getSegmentCenterX() > -262 && t2.getSegmentCenterX() < -260);
    }

    /**
     * Test method for getSegmentCenterY.
     */
    public void testGetSegmentCenterY() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t1 = new TFlow(a, b, 1, 1,transact);
	TFlow t2 = new TFlow(b, a, 1, 2,transact);
        a.addTFlow (t1);
	b.addTFlow (t2);
	t1.drawSelf(null);
	t2.drawSelf(null);
	assertTrue(t1.getSegmentCenterY() > -262 && t1.getSegmentCenterY() < -260);
	assertTrue(t2.getSegmentCenterY() > 560 && t2.getSegmentCenterY() < 562);
    }

    /**
     * Test method for distanceBetween.
     */
    public void testDistanceBetween() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t = new TFlow(null, null, 1, 1,transact);
	int dist = t.distanceBetween(a, b);
	int pyth = (int)Math.sqrt(Math.pow((double)(100 - 200), (double)2) + 
	    Math.pow((double)(100 - 200), (double)2));
	assertTrue(dist == pyth);
    }

    /**
     * Test method for drawSelf.
     */
    public void testDrawSelf() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t = new TFlow(a, b, 1, 1,transact);
	a.addTFlow(t);
	t.drawSelf(null);
	assertTrue(t.getSelectionCenter().x == 150);
	assertTrue(t.getSelectionCenter().y == 150);
    }

     /**
     * Test method for calculateAngles.
     */
    public void testCalculateAngles() {
	TFlow t = new TFlow(null, null, 1, 1,transact);
	double [] angles = t.calculateAngles(new Point(100, 100), 
					     new Point(200, 200));
	assertTrue(angles[0] > 0.99 * Math.PI/4 && angles[0] < 1.01 *
		   Math.PI/4);

	assertTrue(angles [1] > 0.99 * Math.PI/4 - t.ARROW_HEAD_ANGLE && 
	  angles [1] < 1.01 * Math.PI/4 - t.ARROW_HEAD_ANGLE);

	assertTrue(angles [2] > Math.PI - 1.01 * Math.PI/4 - 
		   t.ARROW_HEAD_ANGLE && 
	  angles [2] < Math.PI - 0.99 * Math.PI/4 - t.ARROW_HEAD_ANGLE);
    }

    /**
     * Test method for getOrder.
     */
    public void testGetOrder() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t = new TFlow(a, b, 1, 1,transact);
	a.addTFlow(t);
	assertTrue(t.getOrder() == 1);
    }

    /**
     * Test method for setOrder.
     */
    public void testSetOrder() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t = new TFlow(a, b, 1, 1,transact);
	a.addTFlow(t);
	assertTrue(t.getOrder() == 1);
	t.setOrder(2);
	assertTrue(t.getOrder() == 2);
    }

    /**
     * Test method for isInMainDirection.
     */
    public void testIsInmainDirection() {
	Server a = new Server(new Point(100,100), 0, "aa");
	TFlow t = new TFlow(null, null, 1, 1,transact);
	Point p1 = new Point (100, 80);
	Point p2 = new Point (120, 100);
	Point p3 = new Point (100, 120);
	Point p4 = new Point (80, 100);
	assertTrue(t.isInMainDirection(a, p1) == TFlow.NORTH);
	assertTrue(t.isInMainDirection(a, p2) == TFlow.EAST);
	assertTrue(t.isInMainDirection(a, p3) == TFlow.SOUTH);
	assertTrue(t.isInMainDirection(a, p4) == TFlow.WEST);
	Point p5 = new Point (110, 110);
 	assertTrue(t.isInMainDirection(a, p5) == TFlow.N_DIR);
    }
    
    /**
     * Test method for isRecursion.
     */
    public void testIsRecursion() {
	Server a = new Server(new Point(100,100), 0, "aa");
	TFlow t1 = new TFlow(a, a, new Point (100, 80), 1, 1,transact);
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t2 = new TFlow(a, b, new Point (100, 80), 1, 1,transact);
	assertTrue(t1.isRecursion());
	assertTrue(!t2.isRecursion());
    }

    /**
     * Test method for getSelected.
     */
    public void testGetSelected() {
	Server a = new Server(new Point(100,100), 0, "aa");
	TFlow t1 = new TFlow(a, a, new Point (100, 80), 1, 1,transact);
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t2 = new TFlow(a, b, new Point (100, 80), 1, 1,transact);
	assertTrue(!t1.getSelected());
	assertTrue(!t2.getSelected());
	t1.setSelected(true);
	t2.setSelected(true);
	assertTrue(t1.getSelected());
	assertTrue(t2.getSelected());
    }

    /**
     * Test method for isInsideSelArea.
     */
    public void testIsInsideSelArea() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");

	/*selection area center = 100, 56*/
	TFlow t1 = new TFlow(a, a, new Point (100, 80), 1, 1,transact);
	t1.drawSelf(null);
	a.addTFlow(t1);
	/*selection area center = 150, 150*/

	TFlow t2 = new TFlow(a, b, 1, 1,transact);
	a.addTFlow(t2);
	t2.drawSelf(null);
       
	assertTrue(t1.isInsideSelArea(new Point (101, 55)));
	assertTrue(t2.isInsideSelArea(new Point (151, 151)));
	assertTrue(!t1.isInsideSelArea(new Point (111, 55)));
	assertTrue(!t2.isInsideSelArea(new Point (171, 151)));
    }

    
    /**
     * Test method for getShowSelectionArea.
     */
    public void testGetShowSelectionArea() {
	TFlow t = new TFlow(null, null, 1, 1,transact);
	assertTrue(t.getShowSelectionArea());
	t.setShowSelectionArea(false);
	assertTrue(!t.getShowSelectionArea());
    }

    /**
     * Test method for directionName.
     */
    public void testDirectionName() {
	assertTrue(TFlow.directionName(TFlow.NORTH).equals("North"));
	assertTrue(TFlow.directionName(TFlow.EAST).equals("East"));
	assertTrue(TFlow.directionName(TFlow.SOUTH).equals("South"));
	assertTrue(TFlow.directionName(TFlow.WEST).equals("West"));
	assertTrue(TFlow.directionName(TFlow.N_DIR).equals("N_Dir"));
    }

    /**
     * Test method for giveArcDirection.
     */
    public void testGiveArcDirection() {
	Server a = new Server(new Point(100,100), 0, "aa");
	TFlow t = new TFlow(a, a, new Point (100, 80), 1, 1,transact);
	assertTrue(t.giveArcDirection() == TFlow.NORTH);
    }

    /**
     * Test method for setArcDirection.
     */
    public void testSetArcDirection() {
	Server a = new Server(new Point(100,100), 0, "aa");
	TFlow t = new TFlow(a, a, new Point (100, 80), 1, 1,transact);
	assertTrue(t.giveArcDirection() == TFlow.NORTH);
	t.setArcDirection(TFlow.SOUTH);
	assertTrue(t.giveArcDirection() == TFlow.SOUTH);
    }

    /**
     * Test method for fitBorder.
     */
    public void testFitBorder() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");

	TFlow t = new TFlow(a, b, 1, 1,transact);
	int x = t.fitBorder(a, new Point (200, 200)).x;
	int y = t.fitBorder(a, new Point (200, 200)).y;
	assertTrue(x > 116 && x < 119);
	assertTrue(y > 116 && y < 119);
    }

    /**
     * Test method for getHead.
     */
    public void testGetHead() {
	Point p1 = new Point(100,100); 
	Point p2 = new Point(200,200); 
	Server a = new Server(p1, 0, "aa");
	Server b = new Server(p2, 0, "ac");
	TFlow t = new TFlow(a, b, 1, 1,transact);

	Vector v = t.getHead(p1, p2, 15);
	int[] xp =(int[])(v.elementAt(0));
	int[] yp =(int[])(v.elementAt(1));
    
	assertTrue(xp[0] == 200);
	assertTrue(xp[1] == 187);
        assertTrue(xp[2] == 192);
	assertTrue(yp[0] == 200);
	assertTrue(yp[1] == 192);
        assertTrue(yp[2] == 187);
    }	

    /**
     * Test method for isRouteClear.
     */
    public void testIsRouteClear() {
	Server a = new Server(new Point(100, 100), 0, "aa");
	Server b = new Server(new Point(200, 200), 0, "ac");
	TFlow t = new TFlow(a, b, 1, 1,transact);
        assertTrue(!t.isRouteClear(new Point (160, 160)));
	assertTrue(t.isRouteClear(new Point (210, 210)));
    }

    /**
     * Test method for drawArc.
     */
    public void testDrawArc() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");

	TFlow t1 = new TFlow(a, b, 1, 1,transact);
	TFlow t2 = new TFlow(b, a, 1, 2,transact);
	TFlow t3 = new TFlow(a, b, 1, 3,transact);
	TFlow t4 = new TFlow(b, a, 1, 4,transact);
	TFlow t5 = new TFlow(a, b, 1, 5,transact);
	a.addTFlow(t1);
	b.addTFlow(t2);
	a.addTFlow(t3);
	b.addTFlow(t4);
	a.addTFlow(t5);
	t1.drawSelf(null);
	t2.drawSelf(null);
	t3.drawSelf(null);
	t4.drawSelf(null);
	t5.drawSelf(null);	

	assertTrue(t2.getSelectionCenter().x > 156 && 
		   t2.getSelectionCenter().x < 160);
	assertTrue(t2.getSelectionCenter().y > 139 && 
		   t2.getSelectionCenter().y < 143);
	assertTrue(t5.getSelectionCenter().x > 130 && 
		   t5.getSelectionCenter().x < 134);
	assertTrue(t5.getSelectionCenter().y > 164 && 
		   t5.getSelectionCenter().y < 168);
    }

   
    /**
     * Test method for constructor TFlow().
     */
    public void testTFlow() {
  	Server basicServer = new Server(new Point(1000,1000), 0, "aa");
	TFlow basicFlow = new TFlow(basicServer, basicServer,
				    new Point (1000, 980), 1, 2,transact);
	assertTrue(basicFlow.isRecursion());
	assertTrue(basicFlow.giveArcDirection() == TFlow.NORTH);
	assertTrue(basicFlow.getIdentifier() == 1);
	assertTrue(basicFlow.getOrder() == 2);			
    }

    /**
     * Test method for getYmin.
     */
    public void testGetYmin() {
	Point p1 = new Point (234, 345);
	Point p2 = new Point (334, 344);
	assertTrue(TFlow.getYmin(p1, p2).y == 344);
	assertTrue(TFlow.getYmin(p1, p2).x == 334);
    }

    /**
     * Test method for getYmax.
     */
    public void testGetYmax() {
	Point p1 = new Point (234, 345);
	Point p2 = new Point (234, 344);
	assertTrue(TFlow.getYmax(p1, p2).y == 345);
	assertTrue(TFlow.getYmax(p1, p2).x == 234);
    }

    /**
     * Test method for getXmin.
     */
    public void testGetXmin() {
	Point p1 = new Point (235, 344);
	Point p2 = new Point (234, 444);
	assertTrue(TFlow.getXmin(p1, p2).x == 234);
	assertTrue(TFlow.getXmin(p1, p2).y == 444);
    }

    /**
     * Test method for getXmax.
     */
    public void testGetXmax() {
	Point p1 = new Point (235, 345);
	Point p2 = new Point (234, 344);
	assertTrue(TFlow.getXmax(p1, p2).x == 235);
	assertTrue(TFlow.getXmax(p1, p2).y == 345);
    }

    /**
     * Test method for getFitPoints.
     */
    public void testGetFitPoints() {    
        Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t1 = new TFlow(a, b, 1, 1,transact);
        TFlow t2 = new TFlow(b, a, 1, 2,transact);
    	TFlow t3 = new TFlow(a, b, 1, 3,transact);
    	TFlow t4 = new TFlow(b, a, 1, 4,transact);
    	TFlow t5 = new TFlow(a, b, 1, 5,transact);
    	a.addTFlow(t1);
    	b.addTFlow(t2);
    	a.addTFlow(t3);
    	b.addTFlow(t4);
    	a.addTFlow(t5);
	t5.drawSelf(null);
	Vector v = t5.getFitPoints(a);
	Point p1 = (Point)v.elementAt(0);
	Point p2 = (Point)v.elementAt(1);
      
	assertTrue(p1.x > 100 && p1.x < 104);
	assertTrue(p1.y > 123 && p1.y < 127);
 	assertTrue(p2.x > 86 && p2.x < 90);
	assertTrue(p2.y > 76 && p2.y < 80);
    }		

    /**
     * Test method for drawRecursiveEdge.
     */
    public void testDrawRecursiveEdge() {
	Server a = new Server(new Point(100,100), 0, "aa");
	TFlow t1 = new TFlow(a, a, new Point (100, 80), 1, 1,transact);
	TFlow t2 = new TFlow(a, a, new Point (100, 80), 1, 1,transact);
	a.addTFlow(t1);
	a.addTFlow(t2);
	assertTrue(t1.isRecursion());
	assertTrue(t2.isRecursion());
	assertTrue(t1.giveArcDirection() == TFlow.NORTH);
	assertTrue(t2.giveArcDirection() == TFlow.NORTH);
	t1.drawSelf(null);
	t2.drawSelf(null);
	
	assertTrue(t1.getSelectionCenter().x == 100);
	assertTrue(t1.getSelectionCenter().y == 54);
	assertTrue(t2.getSelectionCenter().x == 100);
	assertTrue(t2.getSelectionCenter().y == 43);
    }

    /**
     * Test method for getHeightIncrement.
     */
    public void testGetHeightIncrement() {
 	TFlow t = new TFlow(null, null, 1, 1,transact);
        assertTrue(t.getHeightIncrement() == Server.getRadius() * 64 / 135);
    }

    /**
     * Test method for getTransaction.
     */
    public void testGetTransaction() {
 	TFlow t = new TFlow(null, null, 1, 1, transact);
        assertTrue(t.getTransaction() == transact);
    }

    /**
     * Test method for setTransaction.
     */
    public void testSetTransaction() {
 	TFlow t = new TFlow(null, null, 1, 1, transact);
        assertTrue(t.getTransaction() == transact);
	Transaction transact1 = new Transaction
		("Test", "Slogan", new Identifier());
	t.setTransaction(transact1);
	assertTrue(t.getTransaction() == transact1);
    }	

    /**
     * Test method for drawDirect.
     */
    public void testDrawDirect() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t1 = new TFlow(a, b, 1, 1,transact);
	TFlow t2 = new TFlow(b, a, 1, 2,transact);
	TFlow t3 = new TFlow(a, b, 1, 3,transact);
	a.addTFlow(t1);
	b.addTFlow(t2);
	a.addTFlow(t3);
	assertTrue(!t1.isRecursion());
	assertTrue(!t2.isRecursion());
	assertTrue(!t3.isRecursion());
	t1.drawSelf(null);
	t2.drawSelf(null);
	t3.drawSelf(null);
	assertTrue(t1.getSelectionCenter().x == 150);
	assertTrue(t1.getSelectionCenter().y == 150);
    }

    /**
     * Test method for drawMultiLine.
     */
    public void testDrawMultiLine() {
	Server a = new Server(new Point(100,100), 0, "aa");
	Server b = new Server(new Point(200,200), 0, "ac");
	TFlow t1 = new TFlow(a, b, 1, 1,transact);
	a.addTFlow(t1);
	assertTrue(!t1.isRecursion());
	assertTrue(t1.getXTurnPoints() == null);
	assertTrue(t1.getYTurnPoints() == null);
	int [] xTp = {130};
	int [] yTp = {50}; 
	t1.setXTurnPoints(xTp);
	t1.setYTurnPoints(yTp);
	assertTrue(t1.getXTurnPoints() != null);
	assertTrue(t1.getYTurnPoints() != null);
	t1.drawSelf(null);
	assertTrue(t1.getSelectionCenter().x == 130);
	assertTrue(t1.getSelectionCenter().y == 50);	
    }

    /**
     * Test method for fitSegment.
     */
    public void testFitSegment() {
	Server s1 = new Server(new Point(100,100), 0, "aa");
	Server s2 = new Server(new Point(200,120), 1, "ac");
	TFlow t1 = new TFlow(s1, s2, 1, 1,transact);
	TFlow t2 = new TFlow(s2, s1, 1, 2,transact);
	s1.addTFlow(t1);
	s2.addTFlow(t2);
	Frame fr = new Frame();
	fr.pack();
	fr.setVisible(true);
	testGraphics = fr.getGraphics();
	fr.dispose();
	fr = null;
	s2.drawSelf(testGraphics, null);
	t1.drawSelf(null);
	t2.drawSelf(null);
	Vector v = t1.getFitPoints(s2);
	Point p1 = (Point)v.elementAt(0);
	Point p2 = (Point)v.elementAt(1);
	v = t2.getFitPoints(s2);
	Point p3 = (Point)v.elementAt(0);
	Point p4 = (Point)v.elementAt(1);
	Point pt1 = t1.fitSegment(s2, TFlow.getXmin(p1, p2), 
				  TFlow.getXmax(p1, p2));
	Point pt2 = t2.fitSegment(s2, TFlow.getXmin(p3, p4), 
				  TFlow.getXmax(p3, p4));
	assertTrue(pt1.x == 163);
	assertTrue(pt1.y == 118);
	assertTrue(pt2.x == 168);
	assertTrue(pt2.y == 108);
    }

    /**
     * Tests method for turnTurnPoints.
     */
    public void testTurnTurnPoints() {
	int[] xCoord;
	int[] yCoord;
	TFlow t = new TFlow(new Server(new Point(), 0, null),
			    null, null, 1, 98,transact);
	assertTrue(t.getXTurnPoints() == null);
	assertTrue(t.getYTurnPoints() == null);
	t.turnTurnPoints();
	assertTrue(t.getXTurnPoints() == null);
	assertTrue(t.getYTurnPoints() == null);
	for (int i=1; i < 6; ++i) {
	    xCoord = new int[i];
	    yCoord = new int[i];
	    for (int j=0; j < xCoord.length; ++j){
		xCoord[j] = j;
		yCoord[j] = j;
	    }
	    t.setXTurnPoints(xCoord);
	    t.setYTurnPoints(yCoord);
	    int[] tmp = t.getXTurnPoints();
	    int[] tmp2 = t.getYTurnPoints();
	    for (int j=0; j < xCoord.length; ++j) {
		assertTrue(tmp[j] == j);
		assertTrue(tmp2[j] == j);
	    }
	    t.turnTurnPoints();
	    tmp = t.getXTurnPoints();
	    tmp2 = t.getYTurnPoints();
	    for (int j = xCoord.length - 1; j > -1; --j) {
		assertTrue(tmp[(xCoord.length - 1) - j] == j);
		assertTrue(tmp2[(xCoord.length - 1) - j] == j);
	    }
	}
    }
}





